import {
  bm25,
  snippet
} from "../../chunks/chunk-LXCOI4A4.js";
import {
  count
} from "../../chunks/chunk-KNXJJ63X.js";
import {
  any,
  array,
  object
} from "../../chunks/chunk-5WLCVBB3.js";
import {
  Builder,
  Select,
  selection
} from "../../chunks/chunk-YZAKZYR7.js";
import {
  and,
  asc,
  desc,
  eq,
  gt,
  gte,
  inArray,
  isNotNull,
  isNull,
  like,
  lt,
  lte,
  ne,
  not,
  or
} from "../../chunks/chunk-LCBVZEPF.js";
import {
  alias
} from "../../chunks/chunk-74QQ64O4.js";
import {
  input
} from "../../chunks/chunk-6YSLV4QG.js";
import {
  getData,
  getTable,
  internalData,
  internalSql,
  sql
} from "../../chunks/chunk-NXCD6SNE.js";
import "../../chunks/chunk-U5RRZUYZ.js";

// src/backend/resolver/EntryResolver.ts
import { EntryFields } from "alinea/core/EntryFields";
import { EntryRow, EntryStatus } from "alinea/core/EntryRow";
import { EntrySearch } from "alinea/core/EntrySearch";
import { Field } from "alinea/core/Field";
import {
  querySource
} from "alinea/core/Graph";
import {
  getExpr,
  hasExpr,
  hasField,
  hasRoot,
  hasWorkspace
} from "alinea/core/Internal";
import { getScope } from "alinea/core/Scope";
import { hasExact } from "alinea/core/util/Checks";
import { entries, fromEntries } from "alinea/core/util/Objects";
import { unreachable } from "alinea/core/util/Types";

// node_modules/rado/dist/core/expr/Include.js
var Include = class {
  [internalData];
  constructor(data) {
    this[internalData] = data;
  }
  #mapFromDriverValue = (value, specs) => {
    const { select, first } = getData(this);
    const parsed = specs.parsesJson ? value : JSON.parse(value);
    if (first) {
      const result = parsed ? select.mapRow({ values: parsed, index: 0, specs }) : null;
      return result;
    }
    if (!parsed)
      return [];
    const rows = parsed;
    const ctx = {
      values: void 0,
      index: 0,
      specs
    };
    for (let i = 0; i < rows.length; i++) {
      ctx.values = rows[i];
      ctx.index = 0;
      rows[i] = select.mapRow(ctx);
    }
    return rows ?? [];
  };
  get [internalSql]() {
    return sql.chunk("emitInclude", getData(this)).mapWith({
      mapFromDriverValue: this.#mapFromDriverValue
    });
  }
};
function include(select) {
  return new Include({ ...getData(select), first: false });
}
((include2) => {
  function one(select) {
    return new Include({ ...getData(select), first: true });
  }
  include2.one = one;
})(include || (include = {}));

// src/backend/resolver/EntryResolver.ts
import { is } from "../util/ORM.js";
import { LinkResolver } from "./LinkResolver.js";
import { ResolveContext } from "./ResolveContext.js";
var orFilter = object({ or: array(any) }).and(hasExact(["or"]));
var andFilter = object({ and: array(any) }).and(hasExact(["and"]));
var builder = new Builder();
var MAX_DEPTH = 999;
var EntryResolver = class {
  constructor(db) {
    this.db = db;
    this.schema = db.config.schema;
    this.scope = getScope(db.config);
  }
  schema;
  scope;
  call(ctx, internal) {
    switch (internal.method) {
      case "snippet":
        return snippet(
          EntrySearch,
          1,
          internal.args[0] && this.expr(ctx, internal.args[0]),
          internal.args[1] && this.expr(ctx, internal.args[1]),
          internal.args[2] && this.expr(ctx, internal.args[2]),
          internal.args[3] && this.expr(ctx, internal.args[3])
        );
      default:
        throw new Error(`Unknown method: "${internal.method}"`);
    }
  }
  expr(ctx, expr) {
    const internal = getExpr(expr);
    switch (internal.type) {
      case "field":
        const name = this.scope.nameOf(expr);
        if (!name)
          throw new Error(`Expression has no name ${expr}`);
        const isEntryField = name === "path" || name === "type";
        if (isEntryField)
          return ctx.Table[name];
        return ctx.Table.data[name];
      case "entryField":
        return ctx.Table[internal.name];
      case "call":
        return this.call(ctx, internal);
      case "value":
        return sql.value(internal.value);
      default:
        unreachable(internal);
    }
  }
  selectCount(ctx, hasSearch) {
    return count(hasSearch ? EntrySearch.rowid : ctx.Table.id).as("count");
  }
  projectTypes(types) {
    return entries(EntryFields).concat(types.flatMap(entries));
  }
  projection(query) {
    return query.select ?? fromEntries(
      (query.type ? this.projectTypes(
        Array.isArray(query.type) ? query.type : [query.type]
      ) : []).concat(entries(EntryFields)).concat(query.include ? entries(query.include) : [])
    );
  }
  selectProjection(ctx, value) {
    if (value && hasExpr(value))
      return this.expr(ctx, value);
    const source = querySource(value);
    if (!source)
      return fromEntries(
        entries(value).map(([key, value2]) => {
          return [key, this.selectProjection(ctx, value2)];
        })
      );
    const related = value;
    const isSingle = this.isSingleResult(related);
    const query = this.query(ctx, related);
    return isSingle ? include.one(query) : include(query);
  }
  select(ctx, query) {
    if (query.count === true)
      return this.selectCount(ctx, Boolean(query.search?.length));
    if (query.select && hasExpr(query.select))
      return this.expr(ctx, query.select);
    const fields = this.projection(query);
    return this.selectProjection(ctx, fromEntries(entries(fields)));
  }
  querySource(ctx, query) {
    const hasSearch = Boolean(query.search?.length);
    const { aliased } = getTable(ctx.Table);
    const cursor = hasSearch ? builder.select(ctx.Table).from(EntrySearch).innerJoin(
      ctx.Table,
      eq(sql`${sql.identifier(aliased)}.rowid`, EntrySearch.rowid)
    ) : builder.select().from(ctx.Table);
    const from = alias(EntryRow, `E${ctx.depth - 1}`);
    switch (querySource(query)) {
      case "parent":
        return cursor.where(eq(ctx.Table.id, from.parentId)).limit(1);
      case "next":
        return cursor.where(
          eq(ctx.Table.parentId, from.parentId),
          gt(ctx.Table.index, from.index)
        ).limit(1);
      case "previous":
        return cursor.where(
          eq(ctx.Table.parentId, from.parentId),
          lt(ctx.Table.index, from.index)
        ).limit(1);
      case "siblings":
        return cursor.where(
          eq(ctx.Table.parentId, from.parentId),
          query.siblings?.includeSelf ? void 0 : ne(ctx.Table.id, from.id)
        );
      case "translations":
        return cursor.where(
          eq(ctx.Table.id, from.id),
          query.translations?.includeSelf ? void 0 : ne(ctx.Table.locale, from.locale)
        );
      case "children":
        const Child = alias(EntryRow, "Child");
        const children = builder.$with("children").as(
          builder.select({
            entryId: Child.id,
            parent: Child.parentId,
            level: sql`0`
          }).from(Child).where(
            eq(Child.id, from.id),
            is(Child.locale, from.locale),
            this.conditionStatus(Child, ctx.status)
          ).unionAll(
            (self) => builder.select({
              entryId: Child.id,
              parent: Child.parentId,
              level: sql`${self.level} + 1`
            }).from(Child).innerJoin(self, eq(self.entryId, Child.parentId)).where(
              is(Child.locale, from.locale),
              this.conditionStatus(Child, ctx.status),
              lt(
                self.level,
                Math.min(query.children?.depth ?? MAX_DEPTH, MAX_DEPTH)
              )
            )
          )
        );
        const childrenIds = builder.withRecursive(children).select(children.entryId).from(children).limit(-1).offset(1);
        return cursor.where(
          inArray(ctx.Table.id, childrenIds),
          is(ctx.Table.locale, from.locale)
        ).orderBy(asc(ctx.Table.index));
      case "parents":
        const Parent = alias(EntryRow, "Parent");
        const parents = builder.$with("parents").as(
          builder.select({
            entryId: Parent.id,
            parent: Parent.parentId,
            level: sql`0`
          }).from(Parent).where(
            eq(Parent.id, from.id),
            is(Parent.locale, from.locale),
            this.conditionStatus(Parent, ctx.status)
          ).unionAll(
            (self) => builder.select({
              entryId: Parent.id,
              parent: Parent.parentId,
              level: sql`${self.level} + 1`
            }).from(Parent).innerJoin(self, eq(self.parent, Parent.id)).where(
              is(Parent.locale, from.locale),
              this.conditionStatus(Parent, ctx.status),
              lt(
                self.level,
                Math.min(query.parents?.depth ?? MAX_DEPTH, MAX_DEPTH)
              )
            )
          )
        );
        const parentIds = builder.withRecursive(parents).select(parents.entryId).from(parents).limit(-1).offset(1);
        return cursor.where(
          inArray(ctx.Table.id, parentIds),
          is(ctx.Table.locale, from.locale)
        ).orderBy(asc(ctx.Table.level));
      default:
        return cursor.orderBy(asc(ctx.Table.index));
    }
  }
  groupBy(ctx, groupBy) {
    return groupBy.map((expr) => {
      return this.expr(ctx, expr);
    });
  }
  orderBy(ctx, orderBy) {
    return orderBy.filter((order) => order.asc || order.desc).map((order) => {
      const expr = this.expr(ctx, order.asc ?? order.desc);
      const collated = order.caseSensitive ? expr : sql`${expr} collate nocase`;
      const direction = order.desc ? desc : asc;
      return direction(collated);
    });
  }
  conditionLocale(Table, locale) {
    if (!locale)
      return sql.value(true);
    if (locale === null)
      return isNull(Table.locale);
    return eq(Table.locale, locale);
  }
  conditionStatus(Table, status) {
    switch (status) {
      case "published":
        return eq(Table.status, EntryStatus.Published);
      case "draft":
        return eq(Table.status, EntryStatus.Draft);
      case "archived":
        return eq(Table.status, EntryStatus.Archived);
      case "preferDraft":
        return Table.active;
      case "preferPublished":
        return Table.main;
      case "all":
        return sql.value(true);
    }
  }
  conditionLocation(Table, location) {
    switch (location.length) {
      case 1:
        return eq(Table.workspace, location[0]);
      case 2:
        return and(
          eq(Table.workspace, location[0]),
          eq(Table.root, location[1])
        );
      case 3:
        return and(
          eq(Table.workspace, location[0]),
          eq(Table.root, location[1]),
          like(Table.parentDir, `/${location[2]}%`)
        );
      default:
        return sql.value(true);
    }
  }
  conditionEntryFields(ctx, query) {
    const workspace = query.workspace && typeof query.workspace === "object" && hasWorkspace(query.workspace) ? this.scope.nameOf(query.workspace) : query.workspace;
    const root = query.root && typeof query.root === "object" && hasRoot(query.root) ? this.scope.nameOf(query.root) : query.root;
    return this.conditionFilter(ctx, {
      _id: query.id,
      _parentId: query.parentId,
      _path: query.path,
      _url: query.url,
      _workspace: workspace,
      _root: root
    });
  }
  conditionSearch(Table, searchTerms) {
    if (!searchTerms?.length)
      return sql.value(true);
    const terms = (Array.isArray(searchTerms) ? searchTerms : [searchTerms]).map((term) => `"${term.replaceAll('"', "")}"*`).join(" AND ");
    return sql`${input(EntrySearch)} match ${input(terms)}`;
  }
  conditionTypes(ctx, types) {
    if (Array.isArray(types)) {
      const names = types.map((type) => this.scope.nameOf(type));
      return inArray(ctx.Table.type, names);
    }
    return eq(ctx.Table.type, this.scope.nameOf(types));
  }
  conditionFilter(ctx, filter) {
    const isOrFilter = orFilter.check(filter);
    if (isOrFilter)
      return or(
        ...filter.or.filter(Boolean).map((filter2) => this.conditionFilter(ctx, filter2))
      );
    const isAndFilter = andFilter.check(filter);
    if (isAndFilter)
      return and(
        ...filter.and.filter(Boolean).map((filter2) => this.conditionFilter(ctx, filter2))
      );
    function filterField(ctx2, name) {
      if (name.startsWith("_")) {
        const entryProp = name.slice(1);
        const key = entryProp;
        if (!(key in ctx2.Table))
          throw new Error(`Unknown field: "${name}"`);
        return ctx2.Table[key];
      }
      return ctx2.Table.data[name];
    }
    const conditions = entries(filter).flatMap(function mapCondition([key, value]) {
      const field = filterField(ctx, key);
      if (typeof value !== "object" || !value)
        return value === void 0 ? [] : [value === null ? isNull(field) : eq(field, value)];
      return entries(value).map(([op, value2]) => {
        switch (op) {
          case "is":
            if (value2 === null)
              return isNull(field);
            return eq(field, value2);
          case "isNot":
            if (value2 === null)
              return isNotNull(field);
            return ne(field, value2);
          case "gt":
            return gt(field, value2);
          case "gte":
            return gte(field, value2);
          case "lt":
            return lt(field, value2);
          case "lte":
            return lte(field, value2);
          case "startsWith":
            return like(field, `${value2}%`);
          case "or":
            if (Array.isArray(value2))
              return or(
                ...value2.map((c) => {
                  return and(...mapCondition([key, c]));
                })
              );
            return and(...mapCondition([key, value2]));
          case "in":
            return inArray(field, value2);
          case "notIn":
            return not(inArray(field, value2));
          default:
            throw new Error(`Unknown filter operator: "${op}"`);
        }
      });
    });
    return and(...conditions);
  }
  query(ctx, query) {
    const { type, filter, skip, take, orderBy, groupBy, first, search } = query;
    ctx = ctx.increaseDepth().none;
    let q = this.querySource(ctx, query);
    const queryData = getData(q);
    let preCondition = queryData.where;
    let condition = and(
      preCondition,
      type ? this.conditionTypes(ctx, type) : void 0,
      this.conditionEntryFields(ctx, query),
      this.conditionLocation(ctx.Table, ctx.location),
      this.conditionStatus(ctx.Table, ctx.status),
      querySource(query) === "translations" ? void 0 : this.conditionLocale(ctx.Table, ctx.locale),
      this.conditionSearch(ctx.Table, search),
      filter && this.conditionFilter(ctx, filter)
    );
    if (skip)
      q = q.offset(skip);
    if (take)
      q = q.limit(take);
    const toSelect = this.select(ctx.select, query);
    let result = new Select({
      ...queryData,
      select: selection(toSelect),
      where: condition
    });
    if (groupBy)
      result = result.groupBy(
        ...this.groupBy(ctx, Array.isArray(groupBy) ? groupBy : [groupBy])
      );
    if (search?.length)
      result = result.orderBy(asc(bm25(EntrySearch, 20, 1)));
    else if (orderBy)
      result = result.orderBy(
        ...this.orderBy(ctx, Array.isArray(orderBy) ? orderBy : [orderBy])
      );
    if (first)
      result = result.limit(1);
    return result;
  }
  isSingleResult(query) {
    return Boolean(
      query.first || query.get || query.parent || query.next || query.previous
    );
  }
  async postField(ctx, interim, field) {
    const shape = Field.shape(field);
    await shape.applyLinks(interim, ctx.linkResolver);
  }
  async postExpr(ctx, interim, expr) {
    if (hasField(expr))
      await this.postField(ctx, interim, expr);
  }
  async postRow(ctx, interim, query) {
    if (!interim)
      return;
    const selected = this.projection(query);
    if (selected && hasExpr(selected))
      return this.postExpr(ctx, interim, selected);
    await Promise.all(
      entries(selected).map(([key, value]) => {
        const source = querySource(value);
        if (source)
          return this.post(ctx, interim[key], value);
        return this.postExpr(ctx, interim[key], value);
      })
    );
  }
  post(ctx, interim, input2) {
    if (input2.count === true)
      return;
    const isSingle = this.isSingleResult(input2);
    if (isSingle)
      return this.postRow(ctx, interim, input2);
    return Promise.all(interim.map((row) => this.postRow(ctx, row, input2)));
  }
  resolve = async (query) => {
    const location = Array.isArray(query.location) ? query.location : query.location && this.scope.locationOf(query.location);
    const ctx = new ResolveContext({
      ...query,
      location
    });
    const dbQuery = this.query(ctx, query);
    const singleResult = this.isSingleResult(query);
    const transact = async (tx) => {
      const rows = await dbQuery.all(tx);
      const linkResolver = new LinkResolver(this, tx, ctx.status);
      const result = singleResult ? rows[0] ?? null : rows;
      if (result)
        await this.post({ linkResolver }, result, query);
      return result;
    };
    if (query.preview) {
      const updated = "entry" in query.preview ? query.preview.entry : void 0;
      if (updated) {
        const result = await this.db.preview(updated, transact);
        if (result)
          return result;
      }
    }
    return this.db.store.transaction(transact);
  };
};
export {
  EntryResolver
};
