import {
  coalesce
} from "../chunks/chunk-LXCOI4A4.js";
import {
  boolean,
  integer,
  json,
  text
} from "../chunks/chunk-GUWMJWRU.js";
import "../chunks/chunk-U7HSJRGD.js";
import {
  Functions,
  index,
  table
} from "../chunks/chunk-74QQ64O4.js";
import {
  input
} from "../chunks/chunk-6YSLV4QG.js";
import {
  getData,
  getField,
  internalConstraint,
  internalData,
  sql
} from "../chunks/chunk-NXCD6SNE.js";
import "../chunks/chunk-U5RRZUYZ.js";

// node_modules/rado/dist/core/Constraint.js
var UniqueConstraint = class _UniqueConstraint {
  [internalData];
  constructor(data) {
    this[internalData] = data;
  }
  on(...columns) {
    const fields = columns.map(getField);
    return new _UniqueConstraint({ ...getData(this), fields });
  }
  nullsNotDistinct() {
    return new _UniqueConstraint({ ...getData(this), nullsNotDistinct: true });
  }
  get [internalConstraint]() {
    const { fields, nullsNotDistinct } = getData(this);
    return sql.join([
      sql`unique`,
      nullsNotDistinct ? sql`nulls not distinct` : void 0,
      sql`(${sql.join(
        fields.map((field) => sql.identifier(field.fieldName)),
        sql`, `
      )})`
    ]);
  }
};
var PrimaryKeyConstraint = class {
  constructor(data) {
    this.data = data;
    this[internalData] = data;
  }
  [internalData];
  get [internalConstraint]() {
    const { fields } = getData(this);
    return sql`primary key (${sql.join(
      fields.map((field) => sql.identifier(field.fieldName)),
      sql`, `
    )})`;
  }
};
function primaryKey(...fields) {
  return new PrimaryKeyConstraint({ fields: fields.map(getField) });
}
var ForeignKeyConstraint = class _ForeignKeyConstraint {
  [internalData];
  constructor(data) {
    this[internalData] = data;
  }
  references(...fields) {
    return new _ForeignKeyConstraint({
      ...getData(this),
      references: fields.map(getField)
    });
  }
  get [internalConstraint]() {
    const { fields, references } = getData(this);
    return sql`foreign key (${sql.join(
      fields.map((field) => sql.identifier(field.fieldName)),
      sql`, `
    )}) references ${sql.identifier(references[0].targetName)} (${sql.join(
      references.map((field) => sql.identifier(field.fieldName)),
      sql`, `
    )})`;
  }
};

// src/core/EntryRow.ts
import { createId } from "./Id.js";
var EntryStatus = /* @__PURE__ */ ((EntryStatus2) => {
  EntryStatus2["Draft"] = "draft";
  EntryStatus2["Published"] = "published";
  EntryStatus2["Archived"] = "archived";
  return EntryStatus2;
})(EntryStatus || {});
var ALT_STATUS = [
  "draft" /* Draft */,
  "archived" /* Archived */
];
var EntryRow = table(
  "Entry",
  {
    // Entry data
    id: text().$default(createId),
    status: text().notNull().$type(),
    title: text().notNull(),
    type: text().notNull(),
    seeded: text(),
    // Hierarchy
    workspace: text().notNull(),
    root: text().notNull(),
    index: text().notNull(),
    parentId: text(),
    locale: text(),
    // Entries from which a new draft can be created are marked as active,
    // there is only one active entry per entryId
    active: boolean().notNull(),
    // Per entry there is one main entry, which is either published, archived or
    // draft
    main: boolean().notNull(),
    path: text().notNull(),
    data: json().notNull().$type(),
    // Computed
    level: integer().notNull(),
    // Amount of parents
    filePath: text().notNull(),
    // Filesystem location
    parentDir: text().notNull(),
    // Filesystem location
    childrenDir: text().notNull(),
    // Filesystem location
    rowHash: text().notNull(),
    fileHash: text().notNull(),
    url: text().notNull(),
    searchableText: text().notNull()
  },
  (EntryRow2) => {
    return {
      primary: primaryKey(EntryRow2.id, EntryRow2.status, EntryRow2.locale),
      rowHash: index().on(EntryRow2.rowHash),
      type: index().on(EntryRow2.type),
      parent: index().on(EntryRow2.parentId),
      url: index().on(EntryRow2.url),
      path: index().on(EntryRow2.path),
      fileIdentifier: index().on(
        EntryRow2.filePath,
        EntryRow2.workspace,
        EntryRow2.root
      ),
      parentDir: index().on(EntryRow2.parentDir),
      childrenDir: index().on(EntryRow2.childrenDir)
    };
  }
);
function concat(...slices) {
  return sql.universal({
    mysql: Functions.concat(...slices),
    default: sql.join(slices.map(input), sql` || `)
  });
}
function entryVersionId(entry = EntryRow) {
  return concat(
    entry.id,
    sql.value("."),
    coalesce(entry.locale, "null"),
    sql.value("."),
    entry.status
  );
}
export {
  ALT_STATUS,
  EntryRow,
  EntryStatus,
  concat,
  entryVersionId
};
