import {
  boolean,
  number,
  object,
  type
} from "../chunks/chunk-5WLCVBB3.js";
import "../chunks/chunk-U5RRZUYZ.js";

// src/core/Graph.ts
import { hasExact } from "./util/Checks.js";
var Graph = class {
  constructor(config, resolver) {
    this.config = config;
    this.#resolver = resolver;
  }
  #resolver;
  find(query) {
    return this.#resolver.resolve(query);
  }
  first(query) {
    return this.#resolver.resolve({ ...query, first: true });
  }
  async get(query) {
    const result = await this.#resolver.resolve({ ...query, get: true });
    if (!result)
      throw new Error("Entry not found");
    return result;
  }
  count(query) {
    return this.#resolver.resolve({ ...query, count: true });
  }
};
var plainObject = type((value) => {
  return value?.constructor === Object;
});
var emptySource = plainObject.and(hasExact([]));
var depthSource = object({ depth: number.optional }).and(hasExact(["depth"]));
var includeSource = object({ includeSelf: boolean.optional }).and(hasExact(["includeSelf"]));
var translationsSource = object({ translations: includeSource });
var childrenSource = object({ children: depthSource });
var parentsSource = object({ parents: depthSource });
var siblingsSource = object({ siblings: includeSource });
var parentSource = object({ parent: emptySource });
var nextSource = object({ next: emptySource });
var prevSource = object({ previous: emptySource });
function querySource(query) {
  if (query?.constructor !== Object)
    return;
  if (translationsSource.check(query))
    return "translations";
  if (childrenSource.check(query))
    return "children";
  if (parentsSource.check(query))
    return "parents";
  if (siblingsSource.check(query))
    return "siblings";
  if (parentSource.check(query))
    return "parent";
  if (nextSource.check(query))
    return "next";
  if (prevSource.check(query))
    return "previous";
}
export {
  Graph,
  querySource
};
