import type { ComponentType } from 'react';
import { HasRoot } from './Internal.js';
import { Label } from './Label.js';
import { Page } from './Page.js';
import { Preview } from './Preview.js';
import { Schema } from './Schema.js';
import { Type } from './Type.js';
import { View } from './View.js';
export interface RootI18n {
    locales: Array<string>;
}
export interface RootMeta {
    contains?: Array<string | Type>;
    icon?: ComponentType;
    i18n?: RootI18n;
    /** Point to a React component used to view this root in the dashboard */
    view?: View<{
        root: RootData;
    }>;
    isMediaRoot?: boolean;
    preview?: Preview;
}
export interface ChildrenDefinition {
    [key: string]: Page;
}
export interface RootData extends RootMeta {
    label: string;
}
export type Root<Children extends ChildrenDefinition = ChildrenDefinition> = Children & HasRoot;
export declare namespace Root {
    function label(root: Root): Label;
    function data(root: Root): RootData;
    function preview(root: Root): Preview | undefined;
    function defaultLocale(root: Root): string | undefined;
    function isRoot(value: any): value is Root;
    function isMediaRoot(root: Root): boolean;
    function validate(root: Root, workspaceLabel: string, schema: Schema): void;
    function referencedViews(root: Root): Array<string>;
}
export interface RootOptions<Children> extends RootMeta {
    children?: Children;
}
export interface RootInternal extends RootOptions<ChildrenDefinition> {
    label: string;
}
export declare function root<Entries extends ChildrenDefinition>(label: string, config?: RootOptions<Entries>): Root<Entries>;
