import "../chunks/chunk-U5RRZUYZ.js";

// src/core/Scope.ts
import { Expr } from "./Expr.js";
import { getExpr, hasExpr } from "./Internal.js";
import { entries } from "./util/Objects.js";
var scopes = /* @__PURE__ */ new WeakMap();
var ENTITY_KEY = "@alinea.Entity";
var EXPR_KEY = "@alinea.Expr";
var Scope = class {
  #keys = /* @__PURE__ */ new Map();
  #paths = /* @__PURE__ */ new Map();
  constructor(config) {
    for (const [workspaceName, workspace] of entries(config.workspaces)) {
      this.#insert(workspace, "Workspace", workspaceName);
      for (const [rootName, root] of entries(workspace)) {
        this.#insert(root, "Root", workspaceName, rootName);
        for (const [pageName, page] of entries(root)) {
          this.#insert(page, "Page", workspaceName, rootName, pageName);
        }
      }
    }
    for (const [typeName, type] of entries(config.schema)) {
      this.#insert(type, "Type", typeName);
      for (const [fieldName, field] of entries(type)) {
        this.#insert(field, "Field", typeName, fieldName);
      }
    }
  }
  locationOf(entity) {
    return this.#paths.get(entity)?.slice(1);
  }
  nameOf(entity) {
    return this.#paths.get(entity)?.at(-1);
  }
  stringify(input) {
    const result = JSON.stringify(input, (key, value) => {
      if (this.#paths.has(value))
        return { [ENTITY_KEY]: this.#paths.get(value)?.join(".") };
      if (value && typeof value === "object" && hasExpr(value))
        return { [EXPR_KEY]: getExpr(value) };
      return value;
    });
    return result;
  }
  parse(input) {
    const result = JSON.parse(input, (key, value) => {
      if (value && typeof value === "object") {
        const props = Object.keys(value);
        if (props.length === 1) {
          const [key2] = props;
          if (key2 === ENTITY_KEY)
            return this.#keys.get(value[key2]);
          else if (key2 === EXPR_KEY)
            return new Expr(value[key2]);
        }
      }
      return value;
    });
    return result;
  }
  #insert(entity, type, ...path) {
    const exists = this.#paths.get(entity);
    if (exists)
      throw new Error(
        `${type} '${path.join(".")}' is already in use at '${exists.slice(1).join(".")}' \u2014 ${type}s must be unique`
      );
    const segments = [type].concat(path);
    const key = segments.join(".");
    this.#keys.set(key, entity);
    this.#paths.set(entity, segments);
  }
};
function getScope(config) {
  if (!scopes.has(config)) {
    scopes.set(config, new Scope(config));
  }
  return scopes.get(config);
}
export {
  Scope,
  getScope
};
