import { File } from '@alinea/iso';
import { StoredRow } from 'alinea/core/Infer';
import { ImagePreviewDetails } from 'alinea/core/media/CreatePreview';
import type { CMS } from './CMS.js';
import { Config } from './Config.js';
import { Status } from './Graph.js';
import { Mutation } from './Mutation.js';
import { Type } from './Type.js';
export interface Transaction {
    (cms: CMS): Promise<Array<Mutation>>;
}
export interface Operation {
    [Operation.Data]: Transaction;
}
export declare class Operation {
    static readonly Data: unique symbol;
    constructor(tx: Transaction);
    protected typeName(config: Config, type: Type): string;
}
export interface UploadQuery {
    file: File | [string, Uint8Array];
    workspace?: string;
    root?: string;
    parentId?: string;
    createPreview?(blob: Blob): Promise<ImagePreviewDetails>;
}
export declare class UploadOperation extends Operation {
    id: string;
    constructor(query: UploadQuery);
}
export declare class DeleteOp extends Operation {
    protected entryIds: Array<string>;
    constructor(entryIds: Array<string>);
}
export interface UpdateQuery<Fields> {
    id: string;
    type?: Type<Fields>;
    status?: Status;
    set?: Partial<StoredRow<Fields>>;
}
export declare class UpdateOperation<Definition> extends Operation {
    constructor(query: UpdateQuery<Definition>);
}
export interface CreateQuery<Fields> {
    type: Type<Fields>;
    workspace?: string;
    root?: string;
    parentId?: string;
    locale?: string;
    set?: Partial<StoredRow<Fields>>;
}
export declare class CreateOperation<Definition> extends Operation {
    id: string;
    constructor(query: CreateQuery<Definition>);
}
