import { EntryEditProps } from 'alinea/dashboard/view/EntryEdit';
import type { ComponentType } from 'react';
import { EntryStatus } from './EntryRow.js';
import { Expr } from './Expr.js';
import { Field } from './Field.js';
import { HasType } from './Internal.js';
import { Label } from './Label.js';
import { SummaryProps } from './media/Summary.js';
import { OrderBy } from './OrderBy.js';
import { Section } from './Section.js';
import { RecordShape } from './shape/RecordShape.js';
import { Expand } from './util/Types.js';
import { View } from './View.js';
export interface EntryUrlMeta {
    status: EntryStatus;
    path: string;
    parentPaths: Array<string>;
    locale?: string | null;
}
export type Type<Definition = object> = Definition & HasType;
type TypeRow<Definition> = Expand<{
    [K in keyof Definition as Definition[K] extends Expr<any> ? K : never]: Definition[K] extends Expr<infer T> ? T : never;
}>;
export declare namespace Type {
    type Infer<Definition> = TypeRow<Definition>;
    function label(type: Type): Label;
    function contains(type: Type): Array<string | Type>;
    function isHidden(type: Type): boolean;
    function shape(type: Type): RecordShape;
    function searchableText(type: Type, value: any): string;
    function sections(type: Type): Section[];
    function isContainer(type: Type): boolean;
    function field(type: Type, name: string): Field | undefined;
    function isType(type: any): type is Type;
    function sharedData(type: Type, entryData: Record<string, unknown>): Record<string, unknown> | undefined;
    function initialValue(type: Type): Record<string, unknown>;
    function validate(type: Type): void;
    function referencedViews(type: Type): Array<string>;
}
export interface FieldsDefinition {
    [key: string]: Field;
}
export interface TypeConfig<Definition> {
    fields: Definition;
    /** Accepts entries of these types as children */
    contains?: Array<string | Type>;
    /** Order children entries in the sidebar content tree */
    orderChildrenBy?: OrderBy | Array<OrderBy>;
    /** Entries do not show up in the sidebar content tree */
    hidden?: true;
    /** An icon (React component) to represent this type in the dashboard */
    icon?: ComponentType;
    /** A React component used to view an entry of this type in the dashboard */
    view?: View<EntryEditProps & {
        type: Type;
    }>;
    /** A React component used to view a row of this type in the dashboard */
    summaryRow?: View<SummaryProps>;
    /** A React component used to view a thumbnail of this type in the dashboard */
    summaryThumb?: View<SummaryProps>;
    entryUrl?: (meta: EntryUrlMeta) => string;
}
export interface TypeInternal extends TypeConfig<FieldsDefinition> {
    label: string;
    allFields: Record<string, Field>;
    sections: Array<Section>;
    shape: RecordShape;
}
/** Create a new type */
export declare function type<Fields extends FieldsDefinition>(label: string, config: TypeConfig<Fields>): Type<Fields>;
export {};
