import {
  useAtomValue
} from "../../chunks/chunk-WF77DMLN.js";
import {
  atom
} from "../../chunks/chunk-OBOPLPUQ.js";
import {
  require_dataloader
} from "../../chunks/chunk-ICLGQ53P.js";
import {
  __toESM
} from "../../chunks/chunk-U5RRZUYZ.js";

// src/dashboard/atoms/EntryAtoms.ts
var import_dataloader = __toESM(require_dataloader(), 1);
import { Entry } from "alinea/core/Entry";
import { getType } from "alinea/core/Internal";
import { MutationType } from "alinea/core/Mutation";
import { Type } from "alinea/core/Type";
import { entryFileName } from "alinea/core/util/EntryFilenames";
import {
  generateKeyBetween,
  generateNKeysBetween
} from "alinea/core/util/FractionalIndexing";
import { entries } from "alinea/core/util/Objects";
import { useMemo } from "react";
import { useDashboard } from "../hook/UseDashboard.js";
import { configAtom } from "./DashboardAtoms.js";
import { graphAtom, useMutate } from "./DbAtoms.js";
import { localeAtom, rootAtom, workspaceAtom } from "./NavigationAtoms.js";
function rootId(rootName) {
  return `@alinea/root-${rootName}`;
}
var visibleTypesAtom = atom((get) => {
  const { schema } = get(configAtom);
  return entries(schema).filter(([_, type]) => !Type.isHidden(type)).map(([name]) => name);
});
async function entryTreeRoot(graph, status, workspace, root, visibleTypes) {
  const children = await graph.find({
    select: Entry.id,
    groupBy: Entry.id,
    orderBy: { asc: Entry.index, caseSensitive: true },
    filter: {
      _active: true,
      _workspace: workspace,
      _root: root,
      _parentId: null,
      _type: { in: visibleTypes }
    },
    status
  });
  return {
    id: rootId(root),
    index: "",
    type: "",
    isFolder: true,
    entries: [],
    children
  };
}
var entryTreeItemLoaderAtom = atom(async (get) => {
  const graph = await get(graphAtom);
  const locale = get(localeAtom);
  const visibleTypes = get(visibleTypesAtom);
  const { schema } = get(configAtom);
  const root = get(rootAtom);
  const workspace = get(workspaceAtom);
  return new import_dataloader.default(async (ids) => {
    const indexed = /* @__PURE__ */ new Map();
    const search = ids.filter((id) => id !== rootId(root.name));
    const data = {
      id: Entry.id,
      type: Entry.type,
      title: Entry.title,
      status: Entry.status,
      locale: Entry.locale,
      workspace: Entry.workspace,
      root: Entry.root,
      path: Entry.path,
      parentPaths: {
        parents: {},
        select: Entry.path
      }
    };
    const rows = await graph.find({
      groupBy: Entry.id,
      select: {
        id: Entry.id,
        index: Entry.index,
        type: Entry.type,
        data,
        translations: {
          translations: {},
          select: data
        }
      },
      id: { in: search },
      status: "preferDraft"
    });
    for (const row of rows) {
      const type = schema[row.type];
      const orderBy = getType(type).orderChildrenBy ?? {
        asc: Entry.index,
        caseSensitive: true
      };
      const children = await graph.find({
        select: {
          id: Entry.id,
          locale: Entry.locale
        },
        groupBy: Entry.id,
        orderBy,
        filter: {
          _parentId: row.id,
          _type: { in: visibleTypes }
        },
        status: "preferDraft"
      });
      const entries2 = [row.data].concat(row.translations);
      const translatedChildren = new Set(
        children.filter((child) => child.locale === locale).map((child) => child.id)
      );
      const untranslated = /* @__PURE__ */ new Set();
      const orderedChildren = children.filter((child) => {
        if (translatedChildren.has(child.id))
          return child.locale === locale;
        if (untranslated.has(child.id))
          return false;
        untranslated.add(child.id);
        return true;
      });
      indexed.set(row.id, {
        id: row.id,
        type: row.type,
        index: row.index,
        entries: entries2,
        children: [...new Set(orderedChildren.map((child) => child.id))]
      });
    }
    const res = [];
    for (const id of ids) {
      if (id === rootId(root.name)) {
        res.push(
          await entryTreeRoot(
            graph,
            "preferDraft",
            workspace.name,
            root.name,
            visibleTypes
          )
        );
        continue;
      }
      const entry = indexed.get(id);
      if (!entry) {
        res.push(void 0);
        continue;
      }
      const typeName = entry.entries[0].type;
      const type = schema[typeName];
      const isFolder = Type.isContainer(type);
      res.push({ ...entry, isFolder });
    }
    return res;
  });
});
var loaderAtom = atom((get) => {
  return { loader: get(entryTreeItemLoaderAtom) };
});
function useEntryTreeProvider() {
  const { loader } = useAtomValue(loaderAtom);
  const mutate = useMutate();
  const { config } = useDashboard();
  return useMemo(() => {
    return {
      onDrop(items, { item: parent, childIndex, insertionIndex }) {
        if (items.length !== 1)
          return;
        const [dropping] = items;
        if (childIndex === null)
          return;
        if (dropping.getParent() !== parent) {
          console.warn("Todo: move entries");
          return;
        }
        const children = parent.getChildren();
        const previous = children[childIndex - 1];
        const previousIndexKey = previous?.getItemData()?.index ?? null;
        const next = children.slice(childIndex).find(
          (entry) => !entry || entry?.getItemData().index !== previousIndexKey
        );
        const nextChildIndex = next ? children.indexOf(next) : void 0;
        const nextIndexKey = next?.getItemData()?.index ?? null;
        try {
          const brokenChildren = children.slice(childIndex, nextChildIndex);
          const newIndexKey = generateKeyBetween(previousIndexKey, nextIndexKey);
          const mutations = [];
          if (brokenChildren.length > 0) {
            const newKeys = generateNKeysBetween(
              newIndexKey,
              nextIndexKey,
              brokenChildren.length
            );
            for (let i = 0; i < brokenChildren.length; i++) {
              const child = brokenChildren[i];
              const correctedIndexKey = newKeys[i];
              for (const entry of child.getItemData().entries) {
                mutations.push({
                  type: MutationType.Order,
                  entryId: entry.id,
                  file: entryFileName(config, entry, entry.parentPaths),
                  index: correctedIndexKey
                });
              }
            }
          }
          for (const entry of dropping.getItemData().entries) {
            mutations.push({
              type: MutationType.Order,
              entryId: entry.id,
              file: entryFileName(config, entry, entry.parentPaths),
              index: newIndexKey
            });
          }
          mutate(mutations, true);
        } catch (err) {
          console.error(err);
        }
      },
      async getItem(id) {
        return await (await loader).clear(id).load(id);
      },
      async getChildren(id) {
        return this.getItem(id).then((item) => item?.children ?? []);
      }
    };
  }, [loader]);
}
export {
  rootId,
  useEntryTreeProvider
};
