import {
  atomFamily,
  unwrap
} from "../../chunks/chunk-ZHH24SIG.js";
import {
  atom
} from "../../chunks/chunk-OBOPLPUQ.js";
import "../../chunks/chunk-U5RRZUYZ.js";

// src/dashboard/atoms/EntryEditorAtoms.ts
import { createYDoc, DOC_KEY, parseYDoc } from "alinea/core/Doc";
import { Entry } from "alinea/core/Entry";
import { EntryStatus } from "alinea/core/EntryRow";
import { Field } from "alinea/core/Field";
import { createId } from "alinea/core/Id";
import { getType } from "alinea/core/Internal";
import { MutationType } from "alinea/core/Mutation";
import { Root } from "alinea/core/Root";
import { Type } from "alinea/core/Type";
import { Workspace } from "alinea/core/Workspace";
import { MEDIA_LOCATION } from "alinea/core/media/MediaLocation";
import { encode } from "alinea/core/util/BufferToBase64";
import {
  entryFileName,
  entryFilepath,
  entryInfo,
  entryUrl
} from "alinea/core/util/EntryFilenames";
import { createEntryRow } from "alinea/core/util/EntryRows";
import { entries, fromEntries, values } from "alinea/core/util/Objects";
import * as paths from "alinea/core/util/Paths";
import { FormAtoms } from "alinea/dashboard/atoms/FormAtoms";
import { keepPreviousData } from "alinea/dashboard/util/KeepPreviousData";
import { encodePreviewPayload } from "alinea/preview/PreviewPayload";
import { debounceAtom } from "../util/DebounceAtom.js";
import { clientAtom, configAtom } from "./DashboardAtoms.js";
import {
  dbMetaAtom,
  entryRevisionAtoms,
  graphAtom,
  mutateAtom
} from "./DbAtoms.js";
import { entryEditsAtoms } from "./Edits.js";
import { errorAtom } from "./ErrorAtoms.js";
import { locationAtom } from "./LocationAtoms.js";
import { yAtom } from "./YAtom.js";
var EditMode = /* @__PURE__ */ ((EditMode2) => {
  EditMode2["Editing"] = "editing";
  EditMode2["Diff"] = "diff";
  return EditMode2;
})(EditMode || {});
var EntryTransition = /* @__PURE__ */ ((EntryTransition2) => {
  EntryTransition2[EntryTransition2["SaveDraft"] = 0] = "SaveDraft";
  EntryTransition2[EntryTransition2["SaveTranslation"] = 1] = "SaveTranslation";
  EntryTransition2[EntryTransition2["PublishEdits"] = 2] = "PublishEdits";
  EntryTransition2[EntryTransition2["RestoreRevision"] = 3] = "RestoreRevision";
  EntryTransition2[EntryTransition2["PublishDraft"] = 4] = "PublishDraft";
  EntryTransition2[EntryTransition2["DiscardDraft"] = 5] = "DiscardDraft";
  EntryTransition2[EntryTransition2["ArchivePublished"] = 6] = "ArchivePublished";
  EntryTransition2[EntryTransition2["PublishArchived"] = 7] = "PublishArchived";
  EntryTransition2[EntryTransition2["DeleteFile"] = 8] = "DeleteFile";
  EntryTransition2[EntryTransition2["DeleteArchived"] = 9] = "DeleteArchived";
  return EntryTransition2;
})(EntryTransition || {});
var entryTransitionAtoms = atomFamily((id) => {
  return atom(
    void 0
  );
});
var entryEditorAtoms = atomFamily(
  ({ id, locale: searchLocale }) => {
    return atom(async (get) => {
      if (!id)
        return void 0;
      const config = get(configAtom);
      const client = get(clientAtom);
      const graph = await get(graphAtom);
      let entry = await graph.first({
        select: Entry,
        id,
        locale: searchLocale,
        status: "preferDraft"
      });
      if (!entry) {
        const { searchParams } = get(locationAtom);
        const preferredLanguage = searchParams.get("from");
        entry = await graph.first({
          select: Entry,
          locale: preferredLanguage ?? void 0,
          id,
          status: "preferDraft"
        });
      }
      if (!entry)
        return void 0;
      const entryId = entry.id;
      const locale = entry.locale;
      get(entryRevisionAtoms(entry.id));
      const type = config.schema[entry.type];
      const edits = get(
        entryEditsAtoms({
          id: entry.id,
          locale: entry.locale
        })
      );
      const loadDraft = client.getDraft(entryId).then((draft) => {
        if (draft) {
          edits.applyRemoteUpdate(draft.draft);
          const matches = draft.fileHash === entry.fileHash;
          const isEmpty = !edits.hasData();
          if (!matches || isEmpty) {
            edits.applyEntryData(type, entry.data);
          }
        } else {
          edits.applyEntryData(type, entry.data);
        }
      }).catch(() => {
        edits.applyEntryData(type, entry.data);
      });
      if (!edits.hasData())
        await loadDraft;
      const versions = await graph.find({
        select: {
          ...Entry,
          parents: {
            parents: {},
            select: Entry.id
          }
        },
        locale,
        id: entryId,
        status: "all"
      });
      const withParents = await graph.first({
        select: {
          parents: {
            parents: {},
            select: {
              id: Entry.id,
              path: Entry.path
            }
          }
        },
        id: entryId,
        locale,
        status: "preferDraft"
      });
      console.log(withParents);
      const translations = await graph.find({
        select: {
          locale: Entry.locale,
          entryId: Entry.id
        },
        id: entryId,
        filter: {
          _locale: { isNot: locale }
        },
        status: "preferDraft"
      });
      const parentLink = entry.parentId && await graph.first({
        select: Entry.id,
        id: entry.parentId,
        locale,
        status: "preferDraft"
      });
      const parentNeedsTranslation = entry.parentId ? !parentLink : false;
      if (versions.length === 0)
        return void 0;
      const statuses = fromEntries(
        versions.map((version) => [version.status, version])
      );
      const availableStatuses = values(EntryStatus).filter(
        (status) => statuses[status] !== void 0
      );
      return createEntryEditor({
        parents: withParents?.parents ?? [],
        translations,
        parentNeedsTranslation,
        client,
        config,
        entryId,
        versions,
        statuses,
        availableStatuses,
        edits
      });
    });
  },
  (a, b) => a.locale === b.locale && a.id === b.id
);
var showHistoryAtom = atom(false);
function createEntryEditor(entryData) {
  const { config, availableStatuses, edits } = entryData;
  const activeStatus = availableStatuses[0];
  const activeVersion = entryData.statuses[activeStatus];
  const type = config.schema[activeVersion.type];
  const docs = fromEntries(
    entries(entryData.statuses).map(([status, version]) => [
      status,
      createYDoc(type, version)
    ])
  );
  const yDoc = edits.doc;
  const hasChanges = edits.hasChanges;
  const draftEntry = keepPreviousData(
    yAtom(edits.doc.getMap(DOC_KEY), getDraftEntry)
  );
  const editMode = atom("editing" /* Editing */);
  const view = getType(type).view;
  const previewRevision = atom(
    void 0
  );
  const showHistory = atom(
    (get) => get(showHistoryAtom),
    (get, set, value) => {
      set(showHistoryAtom, value);
      if (!value)
        set(previewRevision, void 0);
    }
  );
  const transition = entryTransitionAtoms(activeVersion.id);
  const statusInUrl = atom((get) => {
    const { search } = get(locationAtom);
    const statusInSearch = search.slice(1);
    if (availableStatuses.includes(statusInSearch))
      return statusInSearch;
    return void 0;
  });
  const selectedStatus = atom((get) => {
    return get(statusInUrl) ?? activeStatus;
  });
  function entryFile(entry, parentPaths) {
    return entryFileName(
      config,
      entry,
      parentPaths ?? entryData.parents.map((p) => p.path)
    );
  }
  const transact = atom(
    null,
    async (get, set, options) => {
      const currentTransition = get(transition);
      if (currentTransition)
        await currentTransition.done;
      const currentChanges = get(hasChanges);
      if (options.clearChanges)
        set(hasChanges, false);
      const done = options.action().catch((error) => {
        if (options.clearChanges)
          set(hasChanges, currentChanges);
        set(errorAtom, options.errorMessage, error);
      }).finally(() => {
        set(transition, void 0);
      });
      set(transition, { transition: options.transition, done });
      return done;
    }
  );
  const saveDraft = atom(null, async (get, set) => {
    const update = await encode(edits.getLocalUpdate());
    const entry = await getDraftEntry({
      status: EntryStatus.Published,
      path: activeVersion.path
    });
    const mutation = {
      type: MutationType.Edit,
      locale: activeVersion.locale,
      previousFile: entryFile(activeVersion),
      file: entryFile(entry),
      entryId: activeVersion.id,
      entry,
      update
    };
    return set(transact, {
      clearChanges: true,
      transition: 0 /* SaveDraft */,
      action: () => set(mutateAtom, [mutation]),
      errorMessage: "Could not complete save action, please try again later"
    });
  });
  const saveTranslation = atom(null, async (get, set, locale) => {
    const graph = await get(graphAtom);
    const parentLink = activeVersion.parentId && await graph.get({
      select: Entry.id,
      locale,
      id: activeVersion.parentId,
      status: "preferDraft"
    });
    if (activeVersion.parentId && !parentLink)
      throw new Error("Parent not found");
    const parentData = parentLink ? await graph.get({
      select: {
        entryId: Entry.id,
        path: Entry.path,
        paths: {
          parents: {},
          select: Entry.path
        }
      },
      id: parentLink,
      locale,
      status: "preferDraft"
    }) : void 0;
    if (activeVersion.parentId && !parentData)
      throw new Error("Parent not translated");
    const parentPaths = parentData?.paths ? parentData.paths.concat(parentData.path) : [];
    const entry = await getDraftEntry({
      id: activeVersion.id,
      status: EntryStatus.Published,
      parent: parentData?.entryId,
      parentPaths,
      locale
    });
    const mutation = {
      type: MutationType.Create,
      locale: entry.locale,
      file: entryFile(entry, parentPaths),
      entryId: activeVersion.id,
      entry
    };
    return set(transact, {
      clearChanges: true,
      transition: 1 /* SaveTranslation */,
      action: () => set(mutateAtom, [mutation]),
      errorMessage: "Could not complete translate action, please try again later"
    });
  });
  async function persistSharedFields(graph, entry) {
    const res = [];
    const { i18n } = Root.data(config.workspaces[entry.workspace][entry.root]);
    if (i18n) {
      const shared = Type.sharedData(type, entry.data);
      if (shared) {
        const translations = await graph.find({
          select: {
            ...Entry,
            parentPaths: {
              parents: {},
              select: Entry.path
            }
          },
          id: entry.id,
          locale: entry.locale,
          status: "preferPublished"
        });
        for (const translation of translations) {
          if (translation.locale === entry.locale)
            continue;
          res.push({
            type: MutationType.Patch,
            locale: translation.locale,
            file: entryFile(translation, translation.parentPaths),
            entryId: translation.id,
            patch: shared
          });
        }
      }
    }
    return res;
  }
  const errorsAtom = atom((get) => {
    return get(get(form).errors);
  });
  const confirmErrorsAtom = atom(null, (get) => {
    const errors = get(errorsAtom);
    if (errors.size > 0) {
      let errorMessage = "";
      for (const [path, { field, error }] of errors.entries()) {
        const label = Field.label(field);
        const line = typeof error === "string" ? `${label}: ${error}` : label;
        errorMessage += `
\u2014 ${line}`;
      }
      const message = `These fields contains errors, are you sure you want to publish?${errorMessage}`;
      return confirm(message);
    }
    return true;
  });
  const publishEdits = atom(null, async (get, set) => {
    if (!set(confirmErrorsAtom))
      return;
    const currentFile = entryFile(activeVersion);
    const update = await encode(edits.getLocalUpdate());
    const entry = await getDraftEntry({ status: EntryStatus.Published });
    const mutations = [];
    const editedFile = entryFile(entry);
    mutations.push({
      type: MutationType.Edit,
      previousFile: currentFile,
      file: editedFile,
      entryId: activeVersion.id,
      locale: entry.locale,
      entry,
      update
    });
    const graph = await get(graphAtom);
    mutations.push(...await persistSharedFields(graph, entry));
    return set(transact, {
      clearChanges: true,
      transition: 2 /* PublishEdits */,
      action: () => set(mutateAtom, mutations),
      errorMessage: "Could not complete publish action, please try again later"
    });
  });
  const restoreRevision = atom(null, async (get, set) => {
    const revision = get(previewRevision);
    if (!revision)
      return;
    const data = await get(revisionData(revision));
    if (!data)
      return;
    const { edits: edits2 } = entryData;
    edits2.applyEntryData(type, data);
    const update = await encode(edits2.getLocalUpdate());
    const entry = await getDraftEntry({
      status: EntryStatus.Published,
      path: activeVersion.path
    });
    const editedFile = entryFile(entry);
    const mutation = {
      type: MutationType.Edit,
      previousFile: editedFile,
      file: editedFile,
      entryId: activeVersion.id,
      locale: entry.locale,
      entry,
      update
    };
    return set(transact, {
      clearChanges: true,
      transition: 3 /* RestoreRevision */,
      action: () => set(mutateAtom, [mutation]),
      errorMessage: "Could not complete publish action, please try again later"
    });
  });
  const publishDraft = atom(null, async (get, set) => {
    if (!set(confirmErrorsAtom))
      return;
    const mutations = [
      {
        type: MutationType.Publish,
        status: EntryStatus.Draft,
        entryId: activeVersion.id,
        locale: activeVersion.locale,
        file: entryFile(activeVersion)
      }
    ];
    const entry = entryData.statuses[EntryStatus.Draft];
    const graph = await get(graphAtom);
    mutations.push(...await persistSharedFields(graph, entry));
    return set(transact, {
      transition: 4 /* PublishDraft */,
      action: () => set(mutateAtom, mutations),
      errorMessage: "Could not complete publish action, please try again later"
    });
  });
  const discardDraft = atom(null, (get, set) => {
    const mutation = {
      type: MutationType.Discard,
      entryId: activeVersion.id,
      locale: activeVersion.locale,
      file: entryFile(activeVersion)
    };
    return set(transact, {
      transition: 5 /* DiscardDraft */,
      action: () => set(mutateAtom, [mutation]),
      errorMessage: "Could not complete discard action, please try again later"
    });
  });
  const archivePublished = atom(null, (get, set) => {
    const published = entryData.statuses[EntryStatus.Published];
    const mutation = {
      type: MutationType.Archive,
      entryId: published.id,
      locale: published.locale,
      file: entryFile(published)
    };
    return set(transact, {
      transition: 6 /* ArchivePublished */,
      action: () => set(mutateAtom, [mutation]),
      errorMessage: "Could not complete archive action, please try again later"
    });
  });
  const publishArchived = atom(null, (get, set) => {
    const archived = entryData.statuses[EntryStatus.Archived];
    const mutation = {
      type: MutationType.Publish,
      status: EntryStatus.Archived,
      entryId: archived.id,
      locale: archived.locale,
      file: entryFile(archived)
    };
    return set(transact, {
      transition: 7 /* PublishArchived */,
      action: () => set(mutateAtom, [mutation]),
      errorMessage: "Could not complete publish action, please try again later"
    });
  });
  const deleteMediaLibrary = atom(null, (get, set) => {
    const result = confirm(
      "Are you sure you want to delete this folder and all its files?"
    );
    if (!result)
      return;
    const published = entryData.statuses[EntryStatus.Published];
    const mutations = [
      {
        type: MutationType.Archive,
        entryId: published.id,
        locale: published.locale,
        file: entryFile(published)
      },
      {
        type: MutationType.Remove,
        entryId: published.id,
        locale: published.locale,
        file: entryFile({ ...published, status: EntryStatus.Archived })
      }
    ];
    return set(transact, {
      transition: 9 /* DeleteArchived */,
      action: () => set(mutateAtom, mutations),
      errorMessage: "Could not complete delete action, please try again later"
    });
  });
  const deleteFile = atom(null, (get, set) => {
    const result = confirm("Are you sure you want to delete this file?");
    if (!result)
      return;
    const published = entryData.statuses[EntryStatus.Published];
    const file = published.data;
    const mutation = {
      type: MutationType.FileRemove,
      entryId: published.id,
      locale: null,
      workspace: published.workspace,
      location: MEDIA_LOCATION in file ? file[MEDIA_LOCATION] : file.location,
      file: entryFile(published),
      replace: false
    };
    return set(transact, {
      transition: 8 /* DeleteFile */,
      action: () => set(mutateAtom, [mutation]),
      errorMessage: "Could not complete delete action, please try again later"
    });
  });
  const deleteArchived = atom(null, (get, set) => {
    const archived = entryData.statuses[EntryStatus.Archived];
    const mutation = {
      type: MutationType.Remove,
      entryId: archived.id,
      locale: archived.locale,
      file: entryFile(archived)
    };
    return set(transact, {
      transition: 9 /* DeleteArchived */,
      action: () => set(mutateAtom, [mutation]),
      errorMessage: "Could not complete delete action, please try again later"
    });
  });
  async function getDraftEntry(options = {}) {
    const data = parseYDoc(type, yDoc);
    const status = options.status ?? activeVersion.status;
    const locale = options.locale ?? activeVersion.locale;
    const path = options.path ?? data.path ?? activeVersion.path;
    const id = options.id ?? activeVersion.id;
    const parent = options.parent ?? activeVersion.parentId;
    const parentPaths = options.parentPaths ?? entryData.parents.map((p) => p.path);
    const draftEntry2 = {
      ...activeVersion,
      ...data,
      id,
      parent,
      locale,
      path,
      status
    };
    const filePath = entryFilepath(config, draftEntry2, parentPaths);
    const parentDir = paths.dirname(filePath);
    const extension = paths.extname(filePath);
    const fileName = paths.basename(filePath, extension);
    const [entryPath] = entryInfo(fileName);
    const childrenDir = paths.join(parentDir, entryPath);
    const urlMeta = {
      locale,
      path,
      status,
      parentPaths
    };
    const url = entryUrl(type, urlMeta);
    return createEntryRow(config, {
      ...draftEntry2,
      parentDir,
      childrenDir,
      filePath,
      url
    });
  }
  const revisionsAtom = atom(async (get) => {
    const client = get(clientAtom);
    const file = entryFile(activeVersion);
    const revisions = await client.revisions(file);
    revisions.sort((a, b) => b.createdAt - a.createdAt);
    return revisions;
  });
  const revisionData = atomFamily(
    (params) => {
      return atom((get) => {
        const client = get(clientAtom);
        return client.revisionData(params.file, params.ref);
      });
    },
    (a, b) => a.file === b.file && a.ref === b.ref
  );
  const revisionDocState = atomFamily(
    (params) => {
      return atom(async (get) => {
        const data = await get(revisionData(params));
        const entry = data ? { ...activeVersion, data } : activeVersion;
        return createYDoc(type, entry);
      });
    },
    (a, b) => a.file === b.file && a.ref === b.ref
  );
  const selectedState = atom((get) => {
    const selected = get(selectedStatus);
    if (selected === activeStatus)
      return edits.doc;
    return docs[selected];
  });
  const activeTitle = yAtom(edits.root, () => edits.root.get("title"));
  const revisionState = atom((get) => {
    const revision = get(previewRevision);
    return revision ? get(revisionDocState(revision)) : void 0;
  });
  const identity = (prev) => prev;
  const currentDoc = atom((get) => {
    return get(unwrap(revisionState, identity)) ?? get(selectedState);
  });
  const form = atom((get) => {
    const doc = get(currentDoc);
    const readOnly = doc !== edits.doc ? true : void 0;
    return new FormAtoms(type, doc.getMap(DOC_KEY), "", { readOnly });
  });
  const yUpdate = debounceAtom(edits.yUpdate, 250);
  const previewPayload = atom(async (get) => {
    const { contentHash } = await get(dbMetaAtom);
    const update = get(yUpdate);
    const status = get(selectedStatus);
    return encodePreviewPayload({
      entryId: activeVersion.id,
      contentHash,
      status,
      update
    });
  });
  const discardEdits = edits.resetChanges;
  const isLoading = edits.isLoading;
  const preview = Root.preview(
    config.workspaces[activeVersion.workspace][activeVersion.root]
  ) ?? Workspace.preview(config.workspaces[activeVersion.workspace]) ?? config.preview;
  const previewToken = atom(async (get) => {
    const client = get(clientAtom);
    return client.previewToken({ entryId: entryData.entryId });
  });
  return {
    ...entryData,
    transition,
    revisionId: createId(),
    activeStatus,
    statusInUrl,
    selectedStatus,
    entryData,
    editMode,
    activeVersion,
    type,
    previewPayload,
    activeTitle,
    hasChanges,
    draftEntry,
    saveDraft,
    publishEdits,
    restoreRevision,
    publishDraft,
    discardDraft,
    archivePublished,
    publishArchived,
    deleteFile,
    deleteMediaLibrary,
    deleteArchived,
    saveTranslation,
    discardEdits,
    isLoading,
    showHistory,
    revisionsAtom,
    previewToken,
    previewRevision,
    preview,
    form,
    view
  };
}
export {
  EditMode,
  EntryTransition,
  createEntryEditor,
  entryEditorAtoms
};
